/*SCRBIN10.C*/
/*
	Copyright Lewis Knox (c) 1989. All rights not expressly granted herein
	are reserved by the author.  User may copy and distribute this file,
	either in its current form or with alterations, as long as he
	does not receive compensation (other than free time on recipient's
	bulletin board system) and as long as he includes this statement
	in the file.  This code may be incorporated into user's programs,
	commercial or otherwise, as long as its authorship is acknowledged
	in the documentation.

	Convert ascii screen files created with a text editor
	to binary screen images. Display images with display.exe.

	This program was written using Microsoft Quick C v. 1.01.

	Last Modified: 8/20/89

	usage: scrbin infile attribute
*/

 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>

 #define TRUE   0
 #define FALSE  -1
 #define MINARG 2           /*minimum number of command line args*/
 #define NORMAL 7           /*default value for text attribute*/
 #define MAXLEN 2000L       /*maximum number of chars in one screen*/

 int attribute = NORMAL;   /*text attribute*/
 char *output;                       /*output filename*/
 FILE *infile, *outfile;             /*input file, output file*/

 int openinf(char *);
 int makeoutf(char *);
 int chekargs(int);
 int parse_args(int, char **);
 int filter(void);
 int writech(unsigned char);

 main(int argc, char *argv[])
 {
	int error;                 /*error indicator returned from functions*/

	/*check for minimum number of args*/
	if ((error = chekargs(argc)) == FALSE)
		exit(FALSE);        /*Terminate on error.*/
	else
		;

	/*parse command line args and open file*/
	if ((error = parse_args(argc, argv)) == FALSE)
		exit(FALSE);   /*If error, close file and terminate.*/
	else
		;

	/*open output file*/
	if ((error = makeoutf(argv[1])) == FALSE)
		exit(FALSE);   /*If error, terminate.*/
	else
		;

	/*filter input file, write to output file, and close files.*/
	if ((error = filter()) == FALSE)
		exit(FALSE);
	else
		exit(TRUE);         /*Terminate*/
 }


 int openinf(char *filearg)      /*Open text input file.*/
 {
	if ((infile = fopen(filearg, "rt")) == NULL) /*open input file*/
	{
		printf("Cannot find file %s.\n", strupr(filearg));
		return(FALSE);   /*Program fails if input file error occurs.*/
	}
	else
		return(TRUE);
 }

 int makeoutf(char *filearg)  /*Open screen output file with default name.*/
 /*filearg is the input filename supplied by user (argv[1]).*/
 {
	char basename[64]; /*base filename taken from filearg*/
	int i = 0;         /*loop counter.*/
	char *subdir;      /*pointer to last subdirectory in filearg*/
	char *drive;       /*pointer to drive name in filarg*/
	char *basest;      /*pointer to beginning of basename in filearg*/

 /*extract base name from input filename.*/

	subdir = strrchr(filearg, '\\');  /*find end of last subdir in filearg*/
	drive = strrchr(filearg, ':');   /*find end of drive name in filearg*/

	/*
	  If filearg contains no drive name and no subdirs, then basename starts
	  at beginning of filearg. Last subdir must follow last drive name, so
	  if subdir exists, basename begins at next char in filearg. If subdir
	  does not exist but drive name does exist, then basename follows drive
	  name. If neither drive name nor subdir exist, then basename begins at
	  filearg.
	*/

	if ((subdir == NULL) && (drive == NULL))
		basest = filearg;
	else if ((subdir == NULL) && (drive != NULL))
		basest = (drive++);
	else if ((subdir != NULL) && (drive == NULL))
		basest = (subdir++);
	else if ((subdir != NULL) && (drive != NULL))  /*strictly speaking,*/
		basest = (subdir++);                      /*this step is redundant*/
	else
		;

	/*basename = pathname + up to first eight chars of filename*/
	for (i = 0; ((i <= (((strlen(filearg)) - (strlen(basest)) + 8)))
		 && (filearg[i] != '.')); i++)
		basename[i] = filearg[i];
	basename[i] = '\0';    /*terminate with null*/
	/*
	  Append scr ext. to basename, and open as output file.
	  Save upper case version of output filename as ouput.
	*/

	output = strupr(strcat(basename, ".SCR"));
	if ((outfile = fopen(output, "wb")) == NULL)
	{
		printf("Error opening output file %s.\n", output);
		return(FALSE); /*Program fails if output file error occurs.*/
	}
	else
		return(TRUE);
 }

 int chekargs(int argnum)
 /*Display usage and fail if correct number of args not given.*/
 {
	if (argnum < MINARG)
	{
		printf("usage: scrbin infile attribute\n");
		return(FALSE);
	}
	else
		return(TRUE);
 }

 int parse_args(int argnum, char *argvec[])
 /*Parse and process first command line argument. Returns 0 on any error.*/
 {
	int result, att;         /*result is error var,
							   att is int value for attribute arg
							 */
	if ((result = openinf(argvec[1])) == FALSE)
	{
		printf("Cannot find file %s.\n", argvec[1]);
		return(FALSE);
	}
	else
		;

	if (argnum > 2)           /*check for legal attribute*/

		/*
			First, convert argv[2] to an int. Then check result
			to see if it is in bounds. Must use an int to test because
			casting to unsigned char cuts off the high order digits that
			signal a value out of bounds. After testing, assign int
			value to attribute, type unsigned char.
		*/

		if (((att = atoi(argvec[2])) > 0) && (att <= 255))
		{
			attribute = att;
			return(TRUE);
		}
		else
		{
			printf("Bad attribute.\n");
			return(FALSE);
		}
	else
		attribute = NORMAL;   /*assign default attribute if none supplied*/
 }

 int filter(void)
 /*
   Read a char from text file, write it to screen file, and then write an
   attribute byte. Replace newline with nulls to end of line.
 */
 {
	char c = '\0';            /*char from text file*/
	int i = 0, j = 0, k = 0;  /*counters*/
	int cerror1 = TRUE;       /*error flags for fclose()*/
	int cerror2 = TRUE;
	int error = TRUE;         /*input file read error flag*/
	long filebyte;            /*length of input file*/

	if ((filebyte = filelength(fileno(infile))) == -1L)
	{
		printf("Error determining file length: using default 2,000 bytes.\n");
		filebyte = MAXLEN;
	}
	else
	{
		if (filebyte >= MAXLEN) /*file length must be less then 2Kb*/
			filebyte = MAXLEN;
		else
			;
	}

	/*read chars and write them along with an attribute byte*/
	for(k = 0; ((c = getc(infile)) != EOF);)
	{if (c != '\n')
		{
			if (writech(c) == FALSE)
				return(FALSE);      /*function terminates on write error*/
			else
				if (writech(attribute) == FALSE)
				return(FALSE);
			else
				;
			k++;        /*increment line position counter*/
		}
		else
			/*
				If char is newline, we write blanks to the end of the
				line. We use k to keep track of our position on the line
				when a newline is encountered, and to calculate the number
				of blanks we need to write. Then, when we have written those
				blanks, we set k, our line position pointer, back to zero,
				since we are ready to start reading a new line.
			*/
		{
			for(j = 0; j <= (79 - k); j++)
			{
				if (writech(' ') == FALSE)
					return(FALSE);
				else
					;
				if (writech(attribute) == FALSE)
					return(FALSE);
				else
					;
			}
			/*
				Set line position counter to beginning of line
				after handling newline.
			*/
			k = 0;
		}
	}

	/*
	  If we detected EOF from input file, then check for error with ferror().
	  If no error, close files. If error, print message and return FALSE.
	*/

	if ((c == EOF) || (error = (ferror(infile)) == TRUE))  /*Input error?*/
	{                                           /*No. Close files.*/
	   cerror1 = fclose(infile);     /*cerror1 = 0 if no error.*/
	   cerror2 = fclose(outfile);    /*cerror2 = 0 if no error.*/
	   if ((cerror1 != TRUE) || (cerror2 != TRUE)) /*error on closing?*/
	   {
		   printf("Error closing files.\n");
		   return(FALSE);
	   }
	   else
		   return(TRUE);
	}
	else                        /*Yes, read error. Print message.*/
	{
	   printf("Error reading input file.\n");
	   return(FALSE);
	}
 }

 int writech(unsigned char c_or_att)
 /*
	Write either screen char or attribute byte to file,
	checking for errors.
 */
 {
	int wrerror;              /*write error flag*/

	/*write a char, and return EOF if an error occurs.*/
	 if ((wrerror = putc(c_or_att, outfile)) == EOF)
	{
		printf("Error writing output file.\n");
		return(FALSE);          /*write error if EOF*/
	}
	else
		;
 }


